function [A] = generateAudioVisualization(fn,H, fbins)
%generateAudioVisualization : Plays music file fn, and plots the mean
%output from filter(s) H which select frequencies in the bins defined by
%fbins
%   fn is a string of the filename for the music file.  The file should be
%   stored in the same directory as the script
%
%   H is an nxn matrix of transfer functions, where h(i,i) is the ith
%   filter transfer function
%
%   fbins is the frequency bins passed in the template main file for this
%   assignment

    %% Input formatting check
    assert(isa(H, 'tf'), 'Input variable H must be a matrix of transfer functions')
    assert(isequal(size(H), [15 15]), 'The size of H must be 15 x 15')
    assert(isfile(fn), 'Filename supplied is not a file.  Check that the music file is in the same directory as this script');
    assert(isequal(size(fbins), [1 14]), 'fbins must be a 15-element vector of frequencies');

    %% Read audio file
    wb = waitbar(0, 'Loading Audio');
    set(wb,'Position', [100 50 300 50])

    [Y, Fs] = audioread(fn);    %Load music file
    y = Y(:,1);                 %Split off only one channel

    fps = 10;   %Predicted FPS for plot update rate
    uR = 1/fps;  %Visual Update Period

    L = Fs*uR;  %Number of frames of visualization
    
    %%  Bode Plot of Frequency Responses 
    waitbar(0, wb, 'Generating Bode Plots');
    
    figure('Name', 'Filter Frequency Response', 'Position', [1000 100 500 500]);
    for i = 1:15
        bodemag(H(i,i));    % Plot Bode magnitude of each filter
        hold on;
        
        waitbar(.25*i/length(H), wb, 'Generating Bode Plots');
    end

    % Plot formatting
    ax = gca;
    ylim([-50,50])
    set(findall(gcf,'type','line'),'linewidth',3)
    waitbar(0, wb, 'Applying Filters');

    %% Filter music
    t = linspace(0, length(y)/Fs, length(y));   % Time axis

    fy = zeros(length(y),length(H));            %Preallocate memory
    for i = 1:length(H)                         
        fy(:,i) = lsim(H(i,i), y, t);           %Compute convolution integral of audio and filter
        waitbar(.25 + .25*i/length(H), wb, 'Applying Filters');
    end

    %% Process Ampliutudes
    amps = zeros(round(length(y)/L), length(H));    %Preallocate memory

    for i = 1:length(y)/(L)
        waitbar(.5 + .5*i/(length(y)/(L)), wb, 'Processing spectrum');
        amps(i,:) = mean(abs(fy((i-1)*L+1:i*L,:))); %Take mean value of filtered date
    end

    close(wb)

    A = audioplayer(y,Fs);
    A.play;                 % Start playing song
    tic                     % start timer with music
    told = 0; fps = 0;

    ymax = max(amps,[],1:2);    %Compute max value for normalization

    F = figure('Name', 'Audio Spectrum', 'Position', [500 100 500 500]);
    set(F, 'Color', [88 89 91]/255);                   %Generate plot, set to update
    h = bar(amps(1,:)/ymax, 'EdgeColor', 'none', 'FaceColor', [1 130/255 0]);
       ylim([0,1]);
       set(gca, 'XTickLabel', {fbins, '>12.5kHz'}, 'Color', [88 89 91]/255, 'XColor', [1,1,1], 'YColor', [1,1,1]);
       xtickangle(90)
       xlabel('Frequency [Hz]');
       ylabel('Normalized Magnitude');
       set(gca,'nextplot','replacechildren'); 

    while A.isplaying
       t = toc;                 %get current time
       if t>=length(y)/Fs
           break
       end

       set(h, 'Ydata', amps(ceil(t/uR),:)/ymax)    %update plot to data at current time

       drawnow;                 %draw updated plot

       fps = fps*.9 + .1/(t-told);
       told = t;
    end
    
    A.pause

end

